<?php

namespace Drp\AdminPanel\Sidebar;

class Sidebar implements SidebarItemInterface
{
    /**
     * Array of all the items within this sidebar.
     *
     * @var SidebarItemInterface[]
     */
    protected $items = [];

    /**
     * Array of all the deferred items within this sidebar.
     *
     * @var SidebarItemInterface[]
     */
    protected $deferredItems = [];

    /**
     * Builds the HTML version of this sidebar item.
     *
     * @param  string $currentPath Current path the admin panel is on.
     * @return string
     */
    public function render($currentPath)
    {
        $html = '<ul class="sidebar-menu">';
        foreach ($this->items as $item) {
            $html .= $item->render($currentPath);
        }
        foreach ($this->deferredItems as $item) {
            $html .= $item->render($currentPath);
        }
        $html .= '</ul>';

        return $html;
    }

    /**
     * Adds a Sidebar Item to this sidebar.
     *
     * @param SidebarItemInterface $item
     */
    public function add(SidebarItemInterface $item, $defer = false)
    {
        if ($defer) {
            $this->deferredItems[] = $item;

            return;
        }
        $this->items[] = $item;
    }

    /**
     * Adds a single sidebar item.
     *
     * @param  string   $icon          Icon to be displayed to the left of the text.
     * @param  string   $text          Text to be displayed.
     * @param  string   $link          Location the sidebar will take you to.
     * @param  callable $showCallable The closure to call when checking to render this item.
     * @param  boolean $openInNewTab  Whether to open the link in a new tab.
     * @param  boolean $defer         Whether to defer the adding of this item until the end.
     * @return self
     */
    public function addItem(
        $icon,
        $text,
        $link,
        $showCallable = null,
        $openInNewTab = false,
        $defer = false
    ) {
        $item = new SidebarSingleItem($icon, $text, $link, $openInNewTab);
        $this->add($item, $defer);

        return $this;
    }

    /**
     * Adds a header sidebar item.
     *
     * @param  string   $icon         Icon to be displayed to the left of the text.
     * @param  string   $text         Text to be displayed.
     * @param  callable $showCallable The closure to call when checking to render this item.
     * @param  boolean  $defer        Whether to defer the adding of this item until the end.
     * @return self
     */
    public function addHeaderItem($icon, $text, $showCallable = null, $defer = false)
    {
        $item = new SidebarHeader($icon, $text, $showCallable);
        $this->add($item, $defer);

        return $this;
    }

    /**
     * Adds a multi level item to the sidebar.
     *
     * @param  string   $icon         Icon to be displayed to the left of the text.
     * @param  string   $text         Text to be displayed.
     * @param  callable $showCallable The closure to call when checking to render this item.
     * @param  boolean  $defer        Whether to defer the adding of this item until the end.
     * @return SidebarMultiLevelItem
     */
    public function addMultiLevelItem($icon, $text, $showCallable = null, $defer = false)
    {
        $multiItem = new SidebarMultiLevelItem($icon, $text, $showCallable);
        $this->add($multiItem, $defer);

        return $multiItem;
    }
}
